@@ -3,8 +3,11 @@
  *     See COPYRIGHT in top-level directory
  */
 
-#include "mpiimpl.h"
 #include "cdesc.h"
+#include <string.h>
+
+int MPIR_F08_MPI_IN_PLACE MPICH_API_PUBLIC;
+int MPIR_F08_MPI_BOTTOM MPICH_API_PUBLIC;
 
 /*
   Convert an array of strings in Fortran Format to an array of strings in C format (i.e., char* a[]).
@@ -54,9 +57,10 @@ extern int MPIR_Fortran_array_of_string_f2c(const char *strs_f, char ***strs_c,
     }
 
     /* Allocate memory for pointers to strings and the strings themself */
-    buf = (char *) MPL_malloc(sizeof(char *) * num_strs + sizeof(char) * (num_chars + num_strs), MPL_MEM_STRINGS);      /* Add \0 for each string */
+    buf = (char *) malloc(sizeof(char *) * num_strs + sizeof(char) * (num_chars + num_strs));   /* Add \0 for each string */
     if (buf == NULL) {
-        MPIR_ERR_SETANDJUMP(mpi_errno, MPI_ERR_OTHER, "**nomem");
+        mpi_errno = MPI_ERR_OTHER;
+        goto fn_fail;
     }
 
     *strs_c = (char **) buf;
@@ -92,3 +96,43 @@ extern int MPIR_Fortran_array_of_string_f2c(const char *strs_f, char ***strs_c,
   fn_fail:
     goto fn_exit;
 }
+
+void *MPIR_F08_get_MPI_STATUS_IGNORE(void)
+{
+    return (void *) MPI_STATUS_IGNORE;
+}
+
+void *MPIR_F08_get_MPI_STATUSES_IGNORE(void)
+{
+    return (void *) MPI_STATUSES_IGNORE;
+}
+
+void *MPIR_F08_get_MPI_ARGV_NULL(void)
+{
+    return (void *) MPI_ARGV_NULL;
+}
+
+void *MPIR_F08_get_MPI_ARGVS_NULL(void)
+{
+    return (void *) MPI_ARGVS_NULL;
+}
+
+void *MPIR_F08_get_MPI_ERRCODES_IGNORE(void)
+{
+    return (void *) MPI_ERRCODES_IGNORE;
+}
+
+void *MPIR_F08_get_MPI_UNWEIGHTED(void)
+{
+    return (void *) MPI_UNWEIGHTED;
+}
+
+void *MPIR_F08_get_MPI_WEIGHTS_EMPTY(void)
+{
+    return (void *) MPI_WEIGHTS_EMPTY;
+}
+
+void *MPIR_F08_get_MPI_BUFFER_AUTOMATIC(void)
+{
+    return (void *) MPI_BUFFER_AUTOMATIC;
+}
