@@ -4,7 +4,6 @@
  */
 
 #include "mpiimpl.h"
-#include "datatype.h"
 
 #if !defined(MPIC_REQUEST_PTR_ARRAY_SIZE)
 #define MPIC_REQUEST_PTR_ARRAY_SIZE 64
@@ -12,8 +11,41 @@
 
 /* These functions are used in the implementation of collective
    operations. They are wrappers around MPID send/recv functions. They do
-   sends/receives by setting the context offset to
-   MPIR_CONTEXT_INTRA_COLL or MPIR_CONTEXT_INTER_COLL. */
+   sends/receives by setting the context offset MPIR_CONTEXT_COLL_OFFSET.
+ */
+
+#ifdef ENABLE_THREADCOMM
+#define DO_MPID_ISEND(buf, count, datatype, dest, tag, comm_ptr, attr, req) \
+    do { \
+        if (comm_ptr->threadcomm) { \
+            mpi_errno = MPIR_Threadcomm_isend_attr(buf, count, datatype, dest, tag, \
+                                                   comm_ptr->threadcomm, attr, req); \
+        } else { \
+            mpi_errno = MPID_Isend(buf, count, datatype, dest, tag, comm_ptr, attr, req); \
+        } \
+    } while (0)
+
+#define DO_MPID_IRECV(buf, count, datatype, source, tag, comm_ptr, attr, req) \
+    do { \
+        if (comm_ptr->threadcomm) { \
+            mpi_errno = MPIR_Threadcomm_irecv_attr(buf, count, datatype, source, tag, \
+                                                   comm_ptr->threadcomm, attr, req, true); \
+        } else { \
+            mpi_errno = MPID_Irecv(buf, count, datatype, source, tag, comm_ptr, attr, req); \
+        } \
+    } while (0)
+
+#else
+#define DO_MPID_ISEND(buf, count, datatype, dest, tag, comm_ptr, attr, req) \
+    do { \
+        mpi_errno = MPID_Isend(buf, count, datatype, dest, tag, comm_ptr, attr, req); \
+    } while (0)
+
+#define DO_MPID_IRECV(buf, count, datatype, source, tag, comm_ptr, attr, req) \
+    do { \
+        mpi_errno = MPID_Irecv(buf, count, datatype, source, tag, comm_ptr, attr, req); \
+    } while (0)
+#endif
 
 int MPIC_Probe(int source, int tag, MPI_Comm comm, MPI_Status * status)
 {
@@ -29,8 +61,7 @@ int MPIC_Probe(int source, int tag, MPI_Comm comm, MPI_Status * status)
 
     MPIR_Comm_get_ptr(comm, comm_ptr);
 
-    attr |= (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
 
     mpi_errno = MPID_Probe(source, tag, comm_ptr, attr, status);
     if (mpi_errno != MPI_SUCCESS)
@@ -49,27 +80,21 @@ int MPIC_Probe(int source, int tag, MPI_Comm comm, MPI_Status * status)
    the global lock is *not* held when this routine is called. (unless we change
    progress_start/end to grab the lock, in which case we must *still* make
    sure that the lock is not held when this routine is called). */
-int MPIC_Wait(MPIR_Request * request_ptr, MPIR_Errflag_t * errflag)
+int MPIC_Wait(MPIR_Request * request_ptr)
 {
     int mpi_errno = MPI_SUCCESS;
 
     MPIR_FUNC_ENTER;
 
-    MPL_DBG_MSG_S(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %s", *errflag ? "TRUE" : "FALSE");
-
-    if (request_ptr->kind == MPIR_REQUEST_KIND__SEND)
-        request_ptr->status.MPI_TAG = 0;
-
-    mpi_errno = MPID_Wait(request_ptr, MPI_STATUS_IGNORE);
+    mpi_errno = MPIR_Wait(request_ptr, MPI_STATUS_IGNORE);
     MPIR_ERR_CHECK(mpi_errno);
 
-    if (request_ptr->kind == MPIR_REQUEST_KIND__RECV)
-        MPIR_Process_status(&request_ptr->status, errflag);
-
-    MPIR_TAG_CLEAR_ERROR_BITS(request_ptr->status.MPI_TAG);
+    if (request_ptr->kind == MPIR_REQUEST_KIND__RECV) {
+        mpi_errno = MPIR_Process_status(&request_ptr->status);
+        MPIR_ERR_CHECK(mpi_errno);
+    }
 
   fn_exit:
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
@@ -102,7 +127,7 @@ int MPIC_Wait(MPIR_Request * request_ptr, MPIR_Errflag_t * errflag)
    this is OK since there is no data that can be received corrupted. */
 
 int MPIC_Send(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest, int tag,
-              MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
+              MPIR_Comm * comm_ptr, MPIR_Errflag_t errflag)
 {
     int mpi_errno = MPI_SUCCESS;
     int attr = 0;
@@ -115,25 +140,21 @@ int MPIC_Send(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest,
         goto fn_exit;
     }
 
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %d", (int) *errflag);
-
     MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", count);
 
-    attr |= (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
+    MPIR_PT2PT_ATTR_SET_ERRFLAG(attr, errflag);
 
-    mpi_errno = MPID_Send_coll(buf, count, datatype, dest, tag, comm_ptr, attr,
-                               &request_ptr, errflag);
+    DO_MPID_ISEND(buf, count, datatype, dest, tag, comm_ptr, attr, &request_ptr);
     MPIR_ERR_CHECK(mpi_errno);
     if (request_ptr) {
-        mpi_errno = MPIC_Wait(request_ptr, errflag);
+        mpi_errno = MPIC_Wait(request_ptr);
         MPIR_ERR_CHECK(mpi_errno);
         MPIR_Request_free(request_ptr);
     }
 
   fn_exit:
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
@@ -142,19 +163,12 @@ int MPIC_Send(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest,
         MPIR_ERR_SET(mpi_errno, MPI_ERR_OTHER, "**nomem");
     if (request_ptr)
         MPIR_Request_free(request_ptr);
-    if (mpi_errno && !*errflag) {
-        if (MPIX_ERR_PROC_FAILED == MPIR_ERR_GET_CLASS(mpi_errno)) {
-            *errflag = MPIR_ERR_PROC_FAILED;
-        } else {
-            *errflag = MPIR_ERR_OTHER;
-        }
-    }
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
 
 int MPIC_Recv(void *buf, MPI_Aint count, MPI_Datatype datatype, int source, int tag,
-              MPIR_Comm * comm_ptr, MPI_Status * status, MPIR_Errflag_t * errflag)
+              MPIR_Comm * comm_ptr, MPI_Status * status)
 {
     int mpi_errno = MPI_SUCCESS;
     int attr = 0;
@@ -169,38 +183,26 @@ int MPIC_Recv(void *buf, MPI_Aint count, MPI_Datatype datatype, int source, int
         goto fn_exit;
     }
 
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %d", (int) *errflag);
-
     MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", count);
 
-    attr |= (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
 
     if (status == MPI_STATUS_IGNORE)
         status = &mystatus;
 
-    mpi_errno = MPID_Recv(buf, count, datatype, source, tag, comm_ptr, attr, status, &request_ptr);
+    DO_MPID_IRECV(buf, count, datatype, source, tag, comm_ptr, attr, &request_ptr);
     MPIR_ERR_CHECK(mpi_errno);
     if (request_ptr) {
-        mpi_errno = MPIC_Wait(request_ptr, errflag);
+        mpi_errno = MPIC_Wait(request_ptr);
         MPIR_ERR_CHECK(mpi_errno);
 
         *status = request_ptr->status;
         mpi_errno = status->MPI_ERROR;
         MPIR_Request_free(request_ptr);
-    } else {
-        MPIR_Process_status(status, errflag);
-
-        MPIR_TAG_CLEAR_ERROR_BITS(status->MPI_TAG);
-    }
-
-    if (MPI_SUCCESS == MPIR_ERR_GET_CLASS(status->MPI_ERROR)) {
-        MPIR_Assert(status->MPI_TAG == tag);
     }
 
   fn_exit:
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
@@ -213,71 +215,10 @@ int MPIC_Recv(void *buf, MPI_Aint count, MPI_Datatype datatype, int source, int
     /* --END ERROR HANDLING-- */
 }
 
-int MPIC_Ssend(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest, int tag,
-               MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
-{
-    int mpi_errno = MPI_SUCCESS;
-    int attr = 0;
-    MPIR_Request *request_ptr = NULL;
-
-    MPIR_FUNC_ENTER;
-
-    /* Return immediately for dummy process */
-    if (unlikely(dest == MPI_PROC_NULL)) {
-        goto fn_exit;
-    }
-
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %d", (int) *errflag);
-
-    MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
-                         "**countneg", "**countneg %d", count);
-
-    attr |= (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
-
-    switch (*errflag) {
-        case MPIR_ERR_NONE:
-            break;
-        case MPIR_ERR_PROC_FAILED:
-            MPIR_TAG_SET_PROC_FAILURE_BIT(tag);
-            break;
-        default:
-            MPIR_TAG_SET_ERROR_BIT(tag);
-    }
-
-    mpi_errno = MPID_Ssend(buf, count, datatype, dest, tag, comm_ptr, attr, &request_ptr);
-    MPIR_ERR_CHECK(mpi_errno);
-    if (request_ptr) {
-        mpi_errno = MPIC_Wait(request_ptr, errflag);
-        MPIR_ERR_CHECK(mpi_errno);
-        MPIR_Request_free(request_ptr);
-    }
-
-  fn_exit:
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
-    MPIR_FUNC_EXIT;
-    return mpi_errno;
-  fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    if (mpi_errno == MPIX_ERR_NOREQ)
-        MPIR_ERR_SET(mpi_errno, MPI_ERR_OTHER, "**nomem");
-    if (request_ptr)
-        MPIR_Request_free(request_ptr);
-    if (mpi_errno && !*errflag) {
-        if (MPIX_ERR_PROC_FAILED == MPIR_ERR_GET_CLASS(mpi_errno)) {
-            *errflag = MPIR_ERR_PROC_FAILED;
-        } else {
-            *errflag = MPIR_ERR_OTHER;
-        }
-    }
-    goto fn_exit;
-    /* --END ERROR HANDLING-- */
-}
-
 int MPIC_Sendrecv(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
                   int dest, int sendtag, void *recvbuf, MPI_Aint recvcount,
                   MPI_Datatype recvtype, int source, int recvtag,
-                  MPIR_Comm * comm_ptr, MPI_Status * status, MPIR_Errflag_t * errflag)
+                  MPIR_Comm * comm_ptr, MPI_Status * status, MPIR_Errflag_t errflag)
 {
     int mpi_errno = MPI_SUCCESS;
     int attr = 0;
@@ -286,15 +227,12 @@ int MPIC_Sendrecv(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype
 
     MPIR_FUNC_ENTER;
 
-    MPL_DBG_MSG_S(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %s", *errflag ? "TRUE" : "FALSE");
-
     MPIR_ERR_CHKANDJUMP1((sendcount < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", sendcount);
     MPIR_ERR_CHKANDJUMP1((recvcount < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", recvcount);
 
-    attr |= (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
 
     if (status == MPI_STATUS_IGNORE)
         status = &mystatus;
@@ -306,8 +244,7 @@ int MPIC_Sendrecv(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype
                             "**nomemreq");
         MPIR_Status_set_procnull(&recv_req_ptr->status);
     } else {
-        mpi_errno = MPID_Irecv(recvbuf, recvcount, recvtype, source, recvtag,
-                               comm_ptr, attr, &recv_req_ptr);
+        DO_MPID_IRECV(recvbuf, recvcount, recvtype, source, recvtag, comm_ptr, attr, &recv_req_ptr);
         MPIR_ERR_CHECK(mpi_errno);
     }
 
@@ -317,14 +254,14 @@ int MPIC_Sendrecv(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype
         MPIR_ERR_CHKANDSTMT(send_req_ptr == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail,
                             "**nomemreq");
     } else {
-        mpi_errno = MPID_Isend_coll(sendbuf, sendcount, sendtype, dest, sendtag,
-                                    comm_ptr, attr, &send_req_ptr, errflag);
+        MPIR_PT2PT_ATTR_SET_ERRFLAG(attr, errflag);
+        DO_MPID_ISEND(sendbuf, sendcount, sendtype, dest, sendtag, comm_ptr, attr, &send_req_ptr);
         MPIR_ERR_CHECK(mpi_errno);
     }
 
-    mpi_errno = MPIC_Wait(send_req_ptr, errflag);
+    mpi_errno = MPIC_Wait(send_req_ptr);
     MPIR_ERR_CHECK(mpi_errno);
-    mpi_errno = MPIC_Wait(recv_req_ptr, errflag);
+    mpi_errno = MPIC_Wait(recv_req_ptr);
     if (mpi_errno)
         MPIR_ERR_POPFATAL(mpi_errno);
 
@@ -342,8 +279,6 @@ int MPIC_Sendrecv(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype
     MPIR_Request_free(recv_req_ptr);
 
   fn_exit:
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
-
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
@@ -362,7 +297,7 @@ int MPIC_Sendrecv(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype
 int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
                           int dest, int sendtag,
                           int source, int recvtag,
-                          MPIR_Comm * comm_ptr, MPI_Status * status, MPIR_Errflag_t * errflag)
+                          MPIR_Comm * comm_ptr, MPI_Status * status, MPIR_Errflag_t errflag)
 {
     int mpi_errno = MPI_SUCCESS;
     MPI_Status mystatus;
@@ -376,25 +311,13 @@ int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
 
     MPIR_FUNC_ENTER;
 
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %d", (int) *errflag);
-
     MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", count);
 
     if (status == MPI_STATUS_IGNORE)
         status = &mystatus;
-    switch (*errflag) {
-        case MPIR_ERR_NONE:
-            break;
-        case MPIR_ERR_PROC_FAILED:
-            MPIR_TAG_SET_PROC_FAILURE_BIT(sendtag);
-            /* fall through */
-        default:
-            MPIR_TAG_SET_ERROR_BIT(sendtag);
-    }
 
-    attr |= (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
 
     if (count > 0 && dest != MPI_PROC_NULL) {
         MPIR_Pack_size(count, datatype, &tmpbuf_size);
@@ -413,7 +336,7 @@ int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
         MPIR_ERR_CHKANDSTMT(rreq == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
         MPIR_Status_set_procnull(&rreq->status);
     } else {
-        mpi_errno = MPID_Irecv(buf, count, datatype, source, recvtag, comm_ptr, attr, &rreq);
+        DO_MPID_IRECV(buf, count, datatype, source, recvtag, comm_ptr, attr, &rreq);
         MPIR_ERR_CHECK(mpi_errno);
     }
 
@@ -422,8 +345,8 @@ int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
         sreq = MPIR_Request_create_complete(MPIR_REQUEST_KIND__SEND);
         MPIR_ERR_CHKANDSTMT(sreq == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
     } else {
-        mpi_errno = MPID_Isend_coll(tmpbuf, actual_pack_bytes, MPI_PACKED, dest,
-                                    sendtag, comm_ptr, attr, &sreq, errflag);
+        MPIR_PT2PT_ATTR_SET_ERRFLAG(attr, errflag);
+        DO_MPID_ISEND(tmpbuf, actual_pack_bytes, MPI_PACKED, dest, sendtag, comm_ptr, attr, &sreq);
         MPIR_ERR_CHECK(mpi_errno);
         if (mpi_errno != MPI_SUCCESS) {
             /* --BEGIN ERROR HANDLING-- */
@@ -435,9 +358,9 @@ int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
         }
     }
 
-    mpi_errno = MPIC_Wait(sreq, errflag);
+    mpi_errno = MPIC_Wait(sreq);
     MPIR_ERR_CHECK(mpi_errno);
-    mpi_errno = MPIC_Wait(rreq, errflag);
+    mpi_errno = MPIC_Wait(rreq);
     MPIR_ERR_CHECK(mpi_errno);
 
     *status = rreq->status;
@@ -455,7 +378,6 @@ int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
 
   fn_exit:
     MPIR_CHKLMEM_FREEALL();
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
@@ -469,10 +391,10 @@ int MPIC_Sendrecv_replace(void *buf, MPI_Aint count, MPI_Datatype datatype,
 }
 
 int MPIC_Isend(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest, int tag,
-               MPIR_Comm * comm_ptr, MPIR_Request ** request_ptr, MPIR_Errflag_t * errflag)
+               MPIR_Comm * comm_ptr, MPIR_Request ** request_ptr, MPIR_Errflag_t errflag)
 {
     int mpi_errno = MPI_SUCCESS;
-    int context_id;
+    int attr = 0;
 
     MPIR_FUNC_ENTER;
 
@@ -484,62 +406,13 @@ int MPIC_Isend(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest,
         goto fn_exit;
     }
 
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %d", (int) *errflag);
-
     MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", count);
 
-    context_id = (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
+    MPIR_PT2PT_ATTR_SET_ERRFLAG(attr, errflag);
 
-    mpi_errno = MPID_Isend_coll(buf, count, datatype, dest, tag, comm_ptr, context_id,
-                                request_ptr, errflag);
-    MPIR_ERR_CHECK(mpi_errno);
-
-  fn_exit:
-    MPIR_FUNC_EXIT;
-    return mpi_errno;
-  fn_fail:
-    if (mpi_errno == MPIX_ERR_NOREQ)
-        MPIR_ERR_SET(mpi_errno, MPI_ERR_OTHER, "**nomem");
-    goto fn_exit;
-}
-
-int MPIC_Issend(const void *buf, MPI_Aint count, MPI_Datatype datatype, int dest, int tag,
-                MPIR_Comm * comm_ptr, MPIR_Request ** request_ptr, MPIR_Errflag_t * errflag)
-{
-    int mpi_errno = MPI_SUCCESS;
-    int context_id;
-
-    MPIR_FUNC_ENTER;
-
-    /* Create a completed request and return immediately for dummy process */
-    if (unlikely(dest == MPI_PROC_NULL)) {
-        *request_ptr = MPIR_Request_create_complete(MPIR_REQUEST_KIND__SEND);
-        MPIR_ERR_CHKANDSTMT((*request_ptr) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail,
-                            "**nomemreq");
-        goto fn_exit;
-    }
-
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %d", (int) *errflag);
-
-    MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
-                         "**countneg", "**countneg %d", count);
-
-    switch (*errflag) {
-        case MPIR_ERR_NONE:
-            break;
-        case MPIR_ERR_PROC_FAILED:
-            MPIR_TAG_SET_PROC_FAILURE_BIT(tag);
-            break;
-        default:
-            MPIR_TAG_SET_ERROR_BIT(tag);
-    }
-
-    context_id = (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Issend(buf, count, datatype, dest, tag, comm_ptr, context_id, request_ptr);
+    DO_MPID_ISEND(buf, count, datatype, dest, tag, comm_ptr, attr, request_ptr);
     MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
@@ -555,7 +428,7 @@ int MPIC_Irecv(void *buf, MPI_Aint count, MPI_Datatype datatype, int source,
                int tag, MPIR_Comm * comm_ptr, MPIR_Request ** request_ptr)
 {
     int mpi_errno = MPI_SUCCESS;
-    int context_id;
+    int attr = 0;
 
     MPIR_FUNC_ENTER;
 
@@ -571,10 +444,9 @@ int MPIC_Irecv(void *buf, MPI_Aint count, MPI_Datatype datatype, int source,
     MPIR_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
                          "**countneg", "**countneg %d", count);
 
-    context_id = (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) ?
-        MPIR_CONTEXT_INTRA_COLL : MPIR_CONTEXT_INTER_COLL;
+    MPIR_PT2PT_ATTR_SET_CONTEXT_OFFSET(attr, MPIR_CONTEXT_COLL_OFFSET);
 
-    mpi_errno = MPID_Irecv(buf, count, datatype, source, tag, comm_ptr, context_id, request_ptr);
+    DO_MPID_IRECV(buf, count, datatype, source, tag, comm_ptr, attr, request_ptr);
     MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
@@ -587,59 +459,44 @@ int MPIC_Irecv(void *buf, MPI_Aint count, MPI_Datatype datatype, int source,
 }
 
 
-int MPIC_Waitall(int numreq, MPIR_Request * requests[], MPI_Status * statuses,
-                 MPIR_Errflag_t * errflag)
+int MPIC_Waitall(int numreq, MPIR_Request * requests[], MPI_Status * statuses)
 {
     int mpi_errno = MPI_SUCCESS;
     int i;
-    MPI_Request request_ptr_array[MPIC_REQUEST_PTR_ARRAY_SIZE];
-    MPI_Request *request_ptrs = request_ptr_array;
     MPI_Status status_static_array[MPIC_REQUEST_PTR_ARRAY_SIZE];
     MPI_Status *status_array = statuses;
-    MPIR_CHKLMEM_DECL(2);
+    MPIR_CHKLMEM_DECL(1);
 
     MPIR_FUNC_ENTER;
 
-    MPL_DBG_MSG_S(MPIR_DBG_PT2PT, TYPICAL, "IN: errflag = %s", *errflag ? "TRUE" : "FALSE");
-
     if (statuses == MPI_STATUSES_IGNORE) {
         status_array = status_static_array;
     }
 
     if (numreq > MPIC_REQUEST_PTR_ARRAY_SIZE) {
-        MPIR_CHKLMEM_MALLOC(request_ptrs, MPI_Request *, numreq * sizeof(MPI_Request), mpi_errno,
-                            "request pointers", MPL_MEM_BUFFER);
-        MPIR_CHKLMEM_MALLOC(status_array, MPI_Status *, numreq * sizeof(MPI_Status), mpi_errno,
-                            "status objects", MPL_MEM_BUFFER);
+        if (statuses == MPI_STATUSES_IGNORE) {
+            MPIR_CHKLMEM_MALLOC(status_array, MPI_Status *, numreq * sizeof(MPI_Status), mpi_errno,
+                                "status objects", MPL_MEM_BUFFER);
+        }
     }
 
-    for (i = 0; i < numreq; ++i) {
-        /* The MPI_TAG field is not set for send operations, so if we want
-         * to check for the error bit in the tag below, we should initialize all
-         * tag fields here. */
-        status_array[i].MPI_TAG = 0;
-        status_array[i].MPI_SOURCE = MPI_PROC_NULL;
-
-        /* Convert the MPIR_Request objects to MPI_Request objects */
-        request_ptrs[i] = requests[i]->handle;
-    }
-
-    mpi_errno = MPIR_Waitall(numreq, request_ptrs, status_array);
+    mpi_errno = MPIR_Waitall(numreq, requests, status_array);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* The errflag value here is for all requests, not just a single one.  If
      * in the future, this function is used for multiple collectives at a
      * single time, we may have to change that. */
     for (i = 0; i < numreq; ++i) {
-        MPIR_Process_status(&status_array[i], errflag);
-
-        MPIR_TAG_CLEAR_ERROR_BITS(status_array[i].MPI_TAG);
+        if (requests[i]->kind == MPIR_REQUEST_KIND__RECV) {
+            mpi_errno = MPIR_Process_status(&status_array[i]);
+            MPIR_ERR_CHECK(mpi_errno);
+        }
+        MPIR_Request_free(requests[i]);
     }
 
   fn_exit:
     if (numreq > MPIC_REQUEST_PTR_ARRAY_SIZE)
         MPIR_CHKLMEM_FREEALL();
-
-    MPL_DBG_MSG_D(MPIR_DBG_PT2PT, TYPICAL, "OUT: errflag = %d", (int) *errflag);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
