@@ -66,36 +66,6 @@ cvars:
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
 
-#ifdef MPICH_DEBUG_PROGRESS
-#define PROGRESS_START \
-    int iter = 0; \
-    MPL_time_t time_start; \
-    if (MPIR_CVAR_DEBUG_PROGRESS_TIMEOUT > 0) { \
-        MPL_wtime(&time_start); \
-    }
-
-#define PROGRESS_CHECK \
-    if (MPIR_CVAR_DEBUG_PROGRESS_TIMEOUT > 0) { \
-        iter++; \
-        if (iter == 0xffff) {\
-            double time_diff = 0.0; \
-            MPL_time_t time_cur; \
-            MPL_wtime(&time_cur); \
-            MPL_wtime_diff(&time_start, &time_cur, &time_diff); \
-            if (time_diff > MPIR_CVAR_DEBUG_PROGRESS_TIMEOUT) { \
-                MPIR_Request_debug(); \
-                MPL_backtrace_show(stdout); \
-            } else { \
-                iter = 0; \
-            } \
-        } \
-    }
-
-#else
-#define PROGRESS_START do {} while (0)
-#define PROGRESS_CHECK do {} while (0)
-#endif
-
 int MPIR_Status_set_cancelled_impl(MPI_Status * status, int flag)
 {
     MPIR_STATUS_SET_CANCEL_BIT(*status, flag ? TRUE : FALSE);
@@ -732,11 +702,11 @@ int MPIR_Wait_state(MPIR_Request * request_ptr, MPI_Status * status, MPID_Progre
 {
     int mpi_errno = MPI_SUCCESS;
 
-    PROGRESS_START;
+    DEBUG_PROGRESS_START;
     while (!MPIR_Request_is_complete(request_ptr)) {
         mpi_errno = MPID_Progress_wait(state);
         MPIR_ERR_CHECK(mpi_errno);
-        PROGRESS_CHECK;
+        DEBUG_PROGRESS_CHECK;
 
         if (unlikely(MPIR_Request_is_anysrc_mismatched(request_ptr))) {
             mpi_errno = MPIR_Request_handle_proc_failed(request_ptr);
@@ -797,13 +767,13 @@ int MPIR_Waitall_state(int count, MPIR_Request * request_ptrs[], MPI_Status arra
 {
     int mpi_errno = MPI_SUCCESS;
 
-    PROGRESS_START;
     if (requests_property & MPIR_REQUESTS_PROPERTY__NO_NULL) {
         for (int i = 0; i < count; ++i) {
+            DEBUG_PROGRESS_START;
             while (!MPIR_Request_is_complete(request_ptrs[i])) {
                 mpi_errno = MPID_Progress_wait(state);
                 MPIR_ERR_CHECK(mpi_errno);
-                PROGRESS_CHECK;
+                DEBUG_PROGRESS_CHECK;
             }
         }
     } else {
@@ -812,13 +782,14 @@ int MPIR_Waitall_state(int count, MPIR_Request * request_ptrs[], MPI_Status arra
                 continue;
             }
             /* wait for ith request to complete */
+            DEBUG_PROGRESS_START;
             while (!MPIR_Request_is_complete(request_ptrs[i])) {
                 /* generalized requests should already be finished */
                 MPIR_Assert(request_ptrs[i]->kind != MPIR_REQUEST_KIND__GREQUEST);
 
                 mpi_errno = MPID_Progress_wait(state);
                 MPIR_ERR_CHECK(mpi_errno);
-                PROGRESS_CHECK;
+                DEBUG_PROGRESS_CHECK;
             }
         }
     }
@@ -993,7 +964,7 @@ int MPIR_Waitany_state(int count, MPIR_Request * request_ptrs[], int *indx, MPI_
 {
     int mpi_errno = MPI_SUCCESS;
 
-    PROGRESS_START;
+    DEBUG_PROGRESS_START;
     for (;;) {
         for (int i = 0; i < count; i++) {
             if ((i + 1) % MPIR_CVAR_REQUEST_POLL_FREQ == 0) {
@@ -1019,7 +990,7 @@ int MPIR_Waitany_state(int count, MPIR_Request * request_ptrs[], int *indx, MPI_
         MPIR_ERR_CHECK(mpi_errno);
         /* Avoid blocking other threads since I am inside an infinite loop */
         MPID_THREAD_CS_YIELD(GLOBAL, MPIR_THREAD_GLOBAL_ALLFUNC_MUTEX);
-        PROGRESS_CHECK;
+        DEBUG_PROGRESS_CHECK;
     }
 
   fn_exit:
@@ -1126,7 +1097,7 @@ int MPIR_Waitsome_state(int incount, MPIR_Request * request_ptrs[],
     MPIR_ERR_CHECK(mpi_errno);
 
     int n_active = 0;
-    PROGRESS_START;
+    DEBUG_PROGRESS_START;
     for (;;) {
         for (int i = 0; i < incount; i++) {
             if ((i + 1) % MPIR_CVAR_REQUEST_POLL_FREQ == 0) {
@@ -1155,7 +1126,7 @@ int MPIR_Waitsome_state(int incount, MPIR_Request * request_ptrs[],
         MPIR_ERR_CHECK(mpi_errno);
         /* Avoid blocking other threads since I am inside an infinite loop */
         MPID_THREAD_CS_YIELD(GLOBAL, MPIR_THREAD_GLOBAL_ALLFUNC_MUTEX);
-        PROGRESS_CHECK;
+        DEBUG_PROGRESS_CHECK;
     }
 
   fn_exit:
