@@ -47,8 +47,8 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_gpu_rma_enabled(const void *ptr)
     if (ENABLE_GPU) {
         MPL_pointer_attr_t attr;
         MPIR_GPU_query_pointer_attr(ptr, &attr);
-        if (MPL_gpu_query_pointer_is_dev(ptr, &attr)) {
-            if (MPIDI_OFI_ENABLE_HMEM && MPIR_GPU_query_pointer_is_strict_dev(ptr, &attr)) {
+        if (MPL_gpu_attr_is_dev(&attr)) {
+            if (MPIDI_OFI_ENABLE_HMEM && MPL_gpu_attr_is_strict_dev(&attr)) {
                 return 1;
             } else
                 return 0;
@@ -241,8 +241,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_do_put(const void *origin_addr,
     /* skip fi_inject path for GPU messages because this path can be
      * very slow */
     if (origin_contig && target_contig &&
-        (origin_bytes <= MPIDI_OFI_global.max_buffered_write &&
-         !MPL_gpu_query_pointer_is_dev(origin_ptr, &attr))) {
+        (origin_bytes <= MPIDI_OFI_global.max_buffered_write && !MPL_gpu_attr_is_dev(&attr))) {
         MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vci).lock);
         MPIDI_OFI_win_cntr_incr(win);
         MPIDI_OFI_CALL_RETRY(fi_inject_write(MPIDI_OFI_WIN(win).ep,
@@ -268,7 +267,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_do_put(const void *origin_addr,
         iov.iov_len = target_bytes;
 
         void *desc = NULL;
-        if (MPIR_GPU_query_pointer_is_strict_dev(iov.iov_base, &attr))
+        if (MPL_gpu_attr_is_strict_dev(&attr))
             MPIDI_OFI_gpu_rma_register(iov.iov_base, iov.iov_len, &attr, win, nic_target, &desc);
 
         msg.desc = desc;
@@ -444,7 +443,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_do_get(void *origin_addr,
         void *desc = NULL;
         MPL_pointer_attr_t attr;
         MPIR_GPU_query_pointer_attr(iov.iov_base, &attr);
-        if (MPIR_GPU_query_pointer_is_strict_dev(iov.iov_base, &attr))
+        if (MPL_gpu_attr_is_strict_dev(&attr))
             MPIDI_OFI_gpu_rma_register(iov.iov_base, iov.iov_len, NULL, win, nic_target, &desc);
 
         msg.desc = desc;
