@@ -8,6 +8,11 @@
 #include <dlfcn.h>
 #include <assert.h>
 
+// We need to give the pre-processor a chance to replace a function, such as:
+// cuMemFree => cuMemFree_v2
+#define STRINGIFY(x) #x
+#define CUDA_SYMBOL_STRING(x) STRINGIFY(x)
+
 #define CUDA_ERR_CHECK(ret) if (unlikely((ret) != cudaSuccess)) goto fn_fail
 #define CU_ERR_CHECK(ret) if (unlikely((ret) != CUDA_SUCCESS)) goto fn_fail
 
@@ -30,10 +35,13 @@ static gpu_free_hook_s *free_hook_chain = NULL;
 
 static CUresult CUDAAPI(*sys_cuMemFree) (CUdeviceptr dptr);
 static cudaError_t CUDARTAPI(*sys_cudaFree) (void *dptr);
+typedef CUresult(*cuMemFree_t) (CUdeviceptr dptr);
+typedef cudaError_t CUDARTAPI(*cudaFree_t) (void *dptr);
 
 static int gpu_mem_hook_init();
+static MPL_initlock_t free_hook_mutex = MPL_INITLOCK_INITIALIZER;
 
-int MPL_gpu_get_dev_count(int *dev_cnt, int *dev_id)
+int MPL_gpu_get_dev_count(int *dev_cnt, int *dev_id, int *subdevice_id)
 {
     int ret = MPL_SUCCESS;
     if (!gpu_initialized) {
@@ -42,6 +50,7 @@ int MPL_gpu_get_dev_count(int *dev_cnt, int *dev_id)
 
     *dev_cnt = device_count;
     *dev_id = max_dev_id;
+    *subdevice_id = 0;
     return ret;
 }
 
@@ -71,7 +80,7 @@ int MPL_gpu_dev_affinity_to_env(int dev_count, char **dev_list, char **env)
     int ret = MPL_SUCCESS;
     memset(affinity_env, 0, MAX_GPU_STR_LEN);
     if (dev_count == 0) {
-        MPL_snprintf(affinity_env, 3, "-1");
+        snprintf(affinity_env, 3, "-1");
     } else {
         int str_offset = 0;
         for (int i = 0; i < dev_count; ++i) {
@@ -87,6 +96,11 @@ int MPL_gpu_dev_affinity_to_env(int dev_count, char **dev_list, char **env)
     return ret;
 }
 
+int MPL_gpu_init_device_mappings(int max_devid, int max_subdev_id)
+{
+    return MPL_SUCCESS;
+}
+
 int MPL_gpu_query_pointer_attr(const void *ptr, MPL_pointer_attr_t * attr)
 {
     int mpl_err = MPL_SUCCESS;
@@ -125,7 +139,23 @@ int MPL_gpu_query_pointer_attr(const void *ptr, MPL_pointer_attr_t * attr)
     goto fn_exit;
 }
 
-int MPL_gpu_ipc_handle_create(const void *ptr, MPL_gpu_ipc_mem_handle_t * ipc_handle)
+int MPL_gpu_attr_is_dev(MPL_pointer_attr_t * attr)
+{
+    return attr->type == MPL_GPU_POINTER_DEV;
+}
+
+int MPL_gpu_attr_is_strict_dev(MPL_pointer_attr_t * attr)
+{
+    return attr->type == MPL_GPU_POINTER_DEV;
+}
+
+int MPL_gpu_query_is_same_dev(int dev1, int dev2)
+{
+    return dev1 == dev2;
+}
+
+int MPL_gpu_ipc_handle_create(const void *ptr, MPL_gpu_device_attr * ptr_attr,
+                              MPL_gpu_ipc_mem_handle_t * ipc_handle)
 {
     int mpl_err = MPL_SUCCESS;
     cudaError_t ret;
@@ -140,7 +170,12 @@ int MPL_gpu_ipc_handle_create(const void *ptr, MPL_gpu_ipc_mem_handle_t * ipc_ha
     goto fn_exit;
 }
 
-int MPL_gpu_ipc_handle_map(MPL_gpu_ipc_mem_handle_t ipc_handle, int dev_id, void **ptr)
+int MPL_gpu_ipc_handle_destroy(const void *ptr, MPL_pointer_attr_t * gpu_attr)
+{
+    return MPL_SUCCESS;
+}
+
+int MPL_gpu_ipc_handle_map(MPL_gpu_ipc_mem_handle_t * ipc_handle, int dev_id, void **ptr)
 {
     int mpl_err = MPL_SUCCESS;
     cudaError_t ret;
@@ -148,7 +183,7 @@ int MPL_gpu_ipc_handle_map(MPL_gpu_ipc_mem_handle_t ipc_handle, int dev_id, void
 
     cudaGetDevice(&prev_devid);
     cudaSetDevice(dev_id);
-    ret = cudaIpcOpenMemHandle(ptr, ipc_handle, cudaIpcMemLazyEnablePeerAccess);
+    ret = cudaIpcOpenMemHandle(ptr, *ipc_handle, cudaIpcMemLazyEnablePeerAccess);
     CUDA_ERR_CHECK(ret);
 
   fn_exit:
@@ -268,15 +303,29 @@ int MPL_gpu_init(int debug_summary)
         goto fn_exit;
     }
 
-    cudaError_t ret = cudaGetDeviceCount(&device_count);
-    CUDA_ERR_CHECK(ret);
+    cudaError_t ret;
+    ret = cudaGetDeviceCount(&device_count);
+    if (ret == cudaErrorNoDevice) {
+        /* call cudaGetLastError() to consume the error */
+        ret = cudaGetLastError();
+        assert(ret == cudaErrorNoDevice);
+        device_count = 0;
+    } else {
+        CUDA_ERR_CHECK(ret);
+    }
 
     if (device_count <= 0) {
         gpu_initialized = 1;
         goto fn_exit;
     }
 
-    char *visible_devices = getenv("CUDA_VISIBLE_DEVICES");
+    MPL_gpu_info.debug_summary = debug_summary;
+    MPL_gpu_info.enable_ipc = true;
+    MPL_gpu_info.ipc_handle_type = MPL_GPU_IPC_HANDLE_SHAREABLE;
+    MPL_gpu_info.specialized_cache = false;
+
+    char *visible_devices;
+    visible_devices = getenv("CUDA_VISIBLE_DEVICES");
     if (visible_devices) {
         local_to_global_map = MPL_malloc(device_count * sizeof(int), MPL_MEM_OTHER);
 
@@ -315,10 +364,12 @@ int MPL_gpu_init(int debug_summary)
      * in cuda, such as cudaFree and cuMemFree, to track user behaviors on
      * the memory buffer and invalidate cached handle/buffer respectively
      * for result correctness. */
+    MPL_initlock_lock(&free_hook_mutex);
     gpu_mem_hook_init();
+    MPL_initlock_unlock(&free_hook_mutex);
     gpu_initialized = 1;
 
-    if (debug_summary) {
+    if (MPL_gpu_info.debug_summary) {
         printf("==== GPU Init (CUDA) ====\n");
         printf("device_count: %d\n", device_count);
         if (visible_devices) {
@@ -344,11 +395,17 @@ int MPL_gpu_finalize(void)
     MPL_free(global_to_local_map);
 
     gpu_free_hook_s *prev;
+    MPL_initlock_lock(&free_hook_mutex);
     while (free_hook_chain) {
         prev = free_hook_chain;
         free_hook_chain = free_hook_chain->next;
         MPL_free(prev);
     }
+    free_hook_chain = NULL;
+    MPL_initlock_unlock(&free_hook_mutex);
+
+    /* Reset initialization state */
+    gpu_initialized = 0;
 
   fn_exit:
     return MPL_SUCCESS;
@@ -374,6 +431,11 @@ int MPL_gpu_get_dev_id_from_attr(MPL_pointer_attr_t * attr)
     return attr->device;
 }
 
+int MPL_gpu_get_root_device(int dev_id)
+{
+    return dev_id;
+}
+
 int MPL_gpu_get_buffer_bounds(const void *ptr, void **pbase, uintptr_t * len)
 {
     int mpl_err = MPL_SUCCESS;
@@ -417,6 +479,7 @@ static void gpu_free_hooks_cb(void *dptr)
     return;
 }
 
+typedef void (*funcptr_t) (void);
 static int gpu_mem_hook_init()
 {
     void *libcuda_handle;
@@ -427,10 +490,11 @@ static int gpu_mem_hook_init()
     libcudart_handle = dlopen("libcudart.so", RTLD_LAZY | RTLD_GLOBAL);
     assert(libcudart_handle);
 
-    sys_cuMemFree = (void *) dlsym(libcuda_handle, "cuMemFree");
+    sys_cuMemFree = (cuMemFree_t) dlsym(libcuda_handle, CUDA_SYMBOL_STRING(cuMemFree));
     assert(sys_cuMemFree);
-    sys_cudaFree = (void *) dlsym(libcudart_handle, "cudaFree");
+    sys_cudaFree = (cudaFree_t) dlsym(libcudart_handle, CUDA_SYMBOL_STRING(cudaFree));
     assert(sys_cudaFree);
+
     return MPL_SUCCESS;
 }
 
@@ -440,12 +504,15 @@ int MPL_gpu_free_hook_register(void (*free_hook) (void *dptr))
     assert(hook_obj);
     hook_obj->free_hook = free_hook;
     hook_obj->next = NULL;
+
+    MPL_initlock_lock(&free_hook_mutex);
     if (!free_hook_chain)
         free_hook_chain = hook_obj;
     else {
         hook_obj->next = free_hook_chain;
         free_hook_chain = hook_obj;
     }
+    MPL_initlock_unlock(&free_hook_mutex);
 
     return MPL_SUCCESS;
 }
@@ -456,8 +523,15 @@ __attribute__ ((visibility("default")))
 CUresult CUDAAPI cuMemFree(CUdeviceptr dptr)
 {
     CUresult result;
+    MPL_initlock_lock(&free_hook_mutex);
+    if (!sys_cuMemFree) {
+        gpu_mem_hook_init();
+    }
+
     gpu_free_hooks_cb((void *) dptr);
     result = sys_cuMemFree(dptr);
+
+    MPL_initlock_unlock(&free_hook_mutex);
     return (result);
 }
 
@@ -467,11 +541,36 @@ __attribute__ ((visibility("default")))
 cudaError_t CUDARTAPI cudaFree(void *dptr)
 {
     cudaError_t result;
+    MPL_initlock_lock(&free_hook_mutex);
+    if (!sys_cudaFree) {
+        gpu_mem_hook_init();
+    }
+
     gpu_free_hooks_cb(dptr);
     result = sys_cudaFree(dptr);
+
+    MPL_initlock_unlock(&free_hook_mutex);
     return result;
 }
 
+int MPL_gpu_fast_memcpy(void *src, MPL_pointer_attr_t * src_attr, void *dest,
+                        MPL_pointer_attr_t * dest_attr, size_t size)
+{
+    return MPL_ERR_GPU_INTERNAL;
+}
+
+int MPL_gpu_imemcpy(void *dest_ptr, void *src_ptr, size_t size, int dev,
+                    MPL_gpu_copy_direction_t dir, MPL_gpu_engine_type_t engine_type,
+                    MPL_gpu_request * req, bool commit)
+{
+    return MPL_ERR_GPU_INTERNAL;
+}
+
+int MPL_gpu_test(MPL_gpu_request * req, int *completed)
+{
+    return MPL_ERR_GPU_INTERNAL;
+}
+
 int MPL_gpu_launch_hostfn(cudaStream_t stream, MPL_gpu_hostfn fn, void *data)
 {
     cudaError_t result;
