@@ -1,12 +1,12 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2003 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #include "mpi.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include "mpitest.h"
 
 /*
  * This test attempts collective communication after a process in
@@ -41,8 +41,7 @@ int main(int argc, char **argv)
         if (errclass == MPIX_ERR_PROC_FAILED) {
             printf(" No Errors\n");
             fflush(stdout);
-        }
-        else {
+        } else {
             fprintf(stderr, "Wrong error code (%d) returned. Expected MPIX_ERR_PROC_FAILED\n",
                     errclass);
         }
@@ -50,8 +49,7 @@ int main(int argc, char **argv)
         if (err) {
             printf(" No Errors\n");
             fflush(stdout);
-        }
-        else {
+        } else {
             fprintf(stderr, "Program reported MPI_SUCCESS, but an error code was expected.\n");
         }
 #endif
@@ -59,5 +57,5 @@ int main(int argc, char **argv)
 
     MPI_Finalize();
 
-    return 0;
+    return MTestReturnValue(err);
 }
