@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2010 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 /* This test is a simplification of the one in perf/manyrma.c that tests
@@ -14,10 +13,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "mpitest.h"
 
 #define MAX_COUNT 65536*4/16
 #define MAX_RMA_SIZE 2  /* 16 in manyrma performance test */
-#define MAX_RUNS 10
+#define MAX_RUNS 8
 #define MAX_ITER_TIME  5.0      /* seconds */
 
 typedef enum { SYNC_NONE = 0,
@@ -51,43 +51,36 @@ int main(int argc, char *argv[])
     int maxSz = MAX_RMA_SIZE;
     double start, end;
 
-    MPI_Init(&argc, &argv);
+    MTest_Init(&argc, &argv);
 
     for (i = 1; i < argc; i++) {
         if (strcmp(argv[i], "-put") == 0) {
             if (rmaChoice == RMA_ALL)
                 rmaChoice = RMA_NONE;
             rmaChoice |= RMA_PUT;
-        }
-        else if (strcmp(argv[i], "-acc") == 0) {
+        } else if (strcmp(argv[i], "-acc") == 0) {
             if (rmaChoice == RMA_ALL)
                 rmaChoice = RMA_NONE;
             rmaChoice |= RMA_ACC;
-        }
-        else if (strcmp(argv[i], "-fence") == 0) {
+        } else if (strcmp(argv[i], "-fence") == 0) {
             if (syncChoice == SYNC_ALL)
                 syncChoice = SYNC_NONE;
             syncChoice |= SYNC_FENCE;
-        }
-        else if (strcmp(argv[i], "-lock") == 0) {
+        } else if (strcmp(argv[i], "-lock") == 0) {
             if (syncChoice == SYNC_ALL)
                 syncChoice = SYNC_NONE;
             syncChoice |= SYNC_LOCK;
-        }
-        else if (strcmp(argv[i], "-pscw") == 0) {
+        } else if (strcmp(argv[i], "-pscw") == 0) {
             if (syncChoice == SYNC_ALL)
                 syncChoice = SYNC_NONE;
             syncChoice |= SYNC_PSCW;
-        }
-        else if (strcmp(argv[i], "-maxsz") == 0) {
+        } else if (strcmp(argv[i], "-maxsz") == 0) {
             i++;
             maxSz = atoi(argv[i]);
-        }
-        else if (strcmp(argv[i], "-maxcount") == 0) {
+        } else if (strcmp(argv[i], "-maxcount") == 0) {
             i++;
             maxCount = atoi(argv[i]);
-        }
-        else {
+        } else {
             fprintf(stderr, "Unrecognized argument %s\n", argv[i]);
             fprintf(stderr,
                     "%s [ -put ] [ -acc ] [ -lock ] [ -fence ] [ -pscw ] [ -maxsz msgsize ]\n",
@@ -112,6 +105,14 @@ int main(int argc, char *argv[])
     MPI_Group_free(&wgroup);
 
     arraysize = maxSz * MAX_COUNT;
+#ifdef USE_WIN_ALLOCATE
+    MPI_Win_allocate(arraysize * sizeof(int), (int) sizeof(int), MPI_INFO_NULL,
+                     MPI_COMM_WORLD, &arraybuffer, &win);
+    if (!arraybuffer) {
+        fprintf(stderr, "Unable to allocate %d words\n", arraysize);
+        MPI_Abort(MPI_COMM_WORLD, 1);
+    }
+#else
     arraybuffer = (int *) malloc(arraysize * sizeof(int));
     if (!arraybuffer) {
         fprintf(stderr, "Unable to allocate %d words\n", arraysize);
@@ -120,6 +121,7 @@ int main(int argc, char *argv[])
 
     MPI_Win_create(arraybuffer, arraysize * sizeof(int), (int) sizeof(int),
                    MPI_INFO_NULL, MPI_COMM_WORLD, &win);
+#endif
 
     if (maxCount > MAX_COUNT) {
         fprintf(stderr, "MaxCount must not exceed %d\n", MAX_COUNT);
@@ -212,86 +214,107 @@ int main(int argc, char *argv[])
 
     MPI_Win_free(&win);
 
+#ifndef USE_WIN_ALLOCATE
+    free(arraybuffer);
+#endif
+
     MPI_Group_free(&accessGroup);
     MPI_Group_free(&exposureGroup);
 
-    /* If we get here without timing out or failing, we succeeded */
-    if (wrank == 0)
-        printf(" No Errors\n");
-
-    MPI_Finalize();
+    MTest_Finalize(0);
     return 0;
 }
 
 
 void RunAccFence(MPI_Win win, int destRank, int cnt, int sz)
 {
-    int k, i, j, one = 1;
+    int k, i, j;
+    int *buf = malloc(sz * sizeof(int));
+
+    for (i = 0; i < sz; i++) {
+        buf[i] = i;
+    }
 
     for (k = 0; k < MAX_RUNS; k++) {
         MPI_Barrier(MPI_COMM_WORLD);
         MPI_Win_fence(0, win);
         j = 0;
         for (i = 0; i < cnt; i++) {
-            MPI_Accumulate(&one, sz, MPI_INT, destRank, j, sz, MPI_INT, MPI_SUM, win);
+            MPI_Accumulate(buf, sz, MPI_INT, destRank, j, sz, MPI_INT, MPI_SUM, win);
             j += sz;
         }
         MPI_Win_fence(0, win);
     }
+
+    free(buf);
 }
 
 void RunAccLock(MPI_Win win, int destRank, int cnt, int sz)
 {
-    int k, i, j, one = 1;
+    int k, i, j;
+    int *buf = malloc(sz * sizeof(int));
+
+    for (i = 0; i < sz; i++) {
+        buf[i] = i;
+    }
 
     for (k = 0; k < MAX_RUNS; k++) {
         MPI_Barrier(MPI_COMM_WORLD);
         MPI_Win_lock(MPI_LOCK_SHARED, destRank, 0, win);
         j = 0;
         for (i = 0; i < cnt; i++) {
-            MPI_Accumulate(&one, sz, MPI_INT, destRank, j, sz, MPI_INT, MPI_SUM, win);
+            MPI_Accumulate(buf, sz, MPI_INT, destRank, j, sz, MPI_INT, MPI_SUM, win);
             j += sz;
         }
         MPI_Win_unlock(destRank, win);
     }
+
+    free(buf);
 }
 
 void RunPutFence(MPI_Win win, int destRank, int cnt, int sz)
 {
-    int k, i, j, one = 1;
+    int k, i, j;
+    int *buf = malloc(sz * sizeof(int));
 
     for (k = 0; k < MAX_RUNS; k++) {
         MPI_Barrier(MPI_COMM_WORLD);
         MPI_Win_fence(0, win);
         j = 0;
         for (i = 0; i < cnt; i++) {
-            MPI_Put(&one, sz, MPI_INT, destRank, j, sz, MPI_INT, win);
+            MPI_Put(buf, sz, MPI_INT, destRank, j, sz, MPI_INT, win);
             j += sz;
         }
         MPI_Win_fence(0, win);
     }
+
+    free(buf);
 }
 
 void RunPutLock(MPI_Win win, int destRank, int cnt, int sz)
 {
-    int k, i, j, one = 1;
+    int k, i, j;
+    int *buf = malloc(sz * sizeof(int));
 
     for (k = 0; k < MAX_RUNS; k++) {
         MPI_Barrier(MPI_COMM_WORLD);
         MPI_Win_lock(MPI_LOCK_SHARED, destRank, 0, win);
         j = 0;
         for (i = 0; i < cnt; i++) {
-            MPI_Put(&one, sz, MPI_INT, destRank, j, sz, MPI_INT, win);
+            MPI_Put(buf, sz, MPI_INT, destRank, j, sz, MPI_INT, win);
             j += sz;
         }
         MPI_Win_unlock(destRank, win);
     }
+
+    free(buf);
 }
 
 void RunPutPSCW(MPI_Win win, int destRank, int cnt, int sz,
                 MPI_Group exposureGroup, MPI_Group accessGroup)
 {
-    int k, i, j, one = 1;
+    int k, i, j;
+    int *buf = malloc(sz * sizeof(int));
 
     for (k = 0; k < MAX_RUNS; k++) {
         MPI_Barrier(MPI_COMM_WORLD);
@@ -299,18 +322,21 @@ void RunPutPSCW(MPI_Win win, int destRank, int cnt, int sz,
         MPI_Win_start(accessGroup, 0, win);
         j = 0;
         for (i = 0; i < cnt; i++) {
-            MPI_Put(&one, sz, MPI_INT, destRank, j, sz, MPI_INT, win);
+            MPI_Put(buf, sz, MPI_INT, destRank, j, sz, MPI_INT, win);
             j += sz;
         }
         MPI_Win_complete(win);
         MPI_Win_wait(win);
     }
+
+    free(buf);
 }
 
 void RunAccPSCW(MPI_Win win, int destRank, int cnt, int sz,
                 MPI_Group exposureGroup, MPI_Group accessGroup)
 {
-    int k, i, j, one = 1;
+    int k, i, j;
+    int *buf = malloc(sz * sizeof(int));
 
     for (k = 0; k < MAX_RUNS; k++) {
         MPI_Barrier(MPI_COMM_WORLD);
@@ -318,10 +344,12 @@ void RunAccPSCW(MPI_Win win, int destRank, int cnt, int sz,
         MPI_Win_start(accessGroup, 0, win);
         j = 0;
         for (i = 0; i < cnt; i++) {
-            MPI_Accumulate(&one, sz, MPI_INT, destRank, j, sz, MPI_INT, MPI_SUM, win);
+            MPI_Accumulate(buf, sz, MPI_INT, destRank, j, sz, MPI_INT, MPI_SUM, win);
             j += sz;
         }
         MPI_Win_complete(win);
         MPI_Win_wait(win);
     }
+
+    free(buf);
 }
